/**
 * Title:       Abstract JMS Queue receiver for ASETS project (ASE Trading System)
 * ClassName:   AbstractQueueReceiver.java
 * Created:     November 22, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Defines an abstract Queue receiver within OpenMQ broker
 */

package asets.api;

import javax.jms.*;


public abstract class AbstractQueueReceiver implements javax.jms.MessageListener
{
    private QueueSession receiveSession = null;
    private QueueConnection connection = null;
    private String userID = null;

    // Constructor used to initialize AbstractQueueReceiver
    public AbstractQueueReceiver(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        // Create a JMS connection factory
        String addressList = imqAddress;
        com.sun.messaging.QueueConnectionFactory queueConnectionFactory = new com.sun.messaging.QueueConnectionFactory();
        queueConnectionFactory.setProperty(com.sun.messaging.ConnectionConfiguration.imqAddressList, addressList);

        // Create a JMS connection
        QueueConnection queueConnection = queueConnectionFactory.createQueueConnection();
        queueConnection.setClientID(userID);

        // Create JMS session object
        QueueSession queueSession = queueConnection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);

        // Create a JMS queue
        Queue destination = queueSession.createQueue(queueName);

        // Create a JMS receiver
        QueueReceiver queueReceiver = queueSession.createReceiver(destination);

        // Set a JMS message listener
        queueReceiver.setMessageListener(this);

        // Intialize the AbstractQueueReceiver application variables
        this.receiveSession = queueSession;
        this.connection = queueConnection;
        this.userID = userID;

        // Start the JMS connection; allows messages to be delivered
        connection.start();
    }

    // Close the JMS connection
    public void close() throws JMSException
    {
        if (connection != null)
            connection.close();
    }

    public QueueSession getSession()
    {
        return this.receiveSession;
    }
    
    public String getUserID()
    {
        return this.userID;
    }
}
