/**
 * Title:       Abstract JMS Queue sender for ASETS project (ASE Trading System)
 * ClassName:   AbstractQueueSenderNoReply.java
 * Created:     November 22, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Defines an abstract Queue sender, within OpenMQ broker
 */

package asets.api;

import javax.jms.*;


public abstract class AbstractQueueSenderNoReply
{
    private QueueSession sendSession = null;
    private QueueSender sender = null;
    private QueueConnection connection = null;
    private String userID = null;

    // Constructor used to initialize AbstractQueueSenderNoReply
    public AbstractQueueSenderNoReply(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        // Create a JMS connection factory
        String addressList = imqAddress;
        com.sun.messaging.QueueConnectionFactory queueConnectionFactory = new com.sun.messaging.QueueConnectionFactory();
        queueConnectionFactory.setProperty(com.sun.messaging.ConnectionConfiguration.imqAddressList, addressList);

        // Create a JMS connection
        QueueConnection queueConnection = queueConnectionFactory.createQueueConnection();
        queueConnection.setClientID(userID);

        // Create JMS session object
        QueueSession queueSession = queueConnection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);

        // Create a JMS queue
        Queue destination = queueSession.createQueue(queueName);

        // Create a JMS sender
        QueueSender queueSender = queueSession.createSender(destination);

        // Intialize the AbstractQueueSenderNoReply application variables
        this.connection = queueConnection;
        this.sendSession = queueSession;
        this.sender = queueSender;
        this.userID = userID;

        // Start the JMS connection; allows messages to be delivered
        connection.start();
    }

    // Close the JMS connection
    public void close() throws JMSException
    {
        if (connection != null)
            connection.close();
    }

    public QueueSession getSession()
    {
        return this.sendSession;
    }

    public QueueSender getSender()
    {
        return this.sender;
    }

    public String getUserID()
    {
        return this.userID;
    }
}
