/**
 * Title:       Abstract JMS Queue sender with reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   AbstractQueueSenderWithReply.java
 * Created:     November 22, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Defines an abstract Queue sender with reply mechanism, within OpenMQ broker
 */

package asets.api;

import javax.jms.*;


public abstract class AbstractQueueSenderWithReply
{
    private QueueSession sendSession = null;
    private QueueConnection connection = null;
    private Queue requestQueue = null;
    private Queue replyQueue = null;
    private String userID = null;

    // Constructor used to initialize AbstractQueueSenderWithReply
    public AbstractQueueSenderWithReply(String imqAddress, String requestQueueName,
            String replyQueueName, String userID) throws JMSException, Exception
    {
        // Create a JMS connection factory
        String addressList = imqAddress;
        com.sun.messaging.QueueConnectionFactory queueConnectionFactory = new com.sun.messaging.QueueConnectionFactory();
        queueConnectionFactory.setProperty(com.sun.messaging.ConnectionConfiguration.imqAddressList, addressList);

        // Create a JMS connection
        QueueConnection queueConnection = queueConnectionFactory.createQueueConnection();
        queueConnection.setClientID(userID);

        // Create JMS session object
        QueueSession queueSession = queueConnection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);

        // Create JMS queues: request and reply, respectively
        Queue requestDestination = queueSession.createQueue(requestQueueName);
        Queue replyDestination = queueSession.createQueue(replyQueueName);

        // Intialize the AbstractQueueSenderWithReply application variables
        this.connection = queueConnection;
        this.sendSession = queueSession;
        this.requestQueue = requestDestination;
        this.replyQueue = replyDestination;
        this.userID = userID;

        // Start the JMS connection; allows messages to be delivered
        connection.start();
    }

    // Close the JMS connection
    public void close() throws JMSException
    {
        if (connection != null)
            connection.close();
    }

    public QueueSession getSession()
    {
        return this.sendSession;
    }

    public Queue getRequestQueue()
    {
        return this.requestQueue;
    }

      public Queue getReplyQueue()
    {
        return this.replyQueue;
    }

    public String getUserID()
    {
        return this.userID;
    }
}
