/**
 * Title:       InstrumentList request sender with reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   InstrumentListSenderWithReply.java
 * Created:     January 9, 2010
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements a Control queue sender, with reply mechanism, within OpenMQ broker, by extending AbstractQueueSenderWithReply class
 */

package asets.api;

import javax.jms.*;


public class InstrumentListSenderWithReply extends AbstractQueueSenderWithReply
{
    public InstrumentListSenderWithReply(String imqAddress, String requestQueueName,
            String replyQueueName, String userID) throws JMSException, Exception
    {
        super(imqAddress, requestQueueName, replyQueueName, userID);
    }

    // Create and send a Control message, waiting for ControlList message reply
    public InstrumentList sendControlGetReply(Control control) throws JMSException, Exception
    {
        // Construct Control request message
        // control.setControlType(Enums.ControlType.INSTRUMENT_LIST_REQUEST);

        ObjectMessage message = this.getSession().createObjectMessage();
        message.setObject(control);

        message.setStringProperty("MSG_TYPE", Enums.MessageType.CONTROL.toString());
        message.setStringProperty("CTRL_TYPE", Enums.ControlType.INSTRUMENT_LIST_REQUEST.toString());
        message.setStringProperty("USER_ID", control.getUserID());
        message.setJMSReplyTo(this.getReplyQueue());

        // Create Control sender and send the message
        QueueSender requestSender = this.getSession().createSender(this.getRequestQueue());
        requestSender.setTimeToLive(30000);     // The request will be valid for 30 sec.
        requestSender.send(message);

        // Wait and process InstrumentList reply
        String filter = "JMSCorrelationID = '" + message.getJMSMessageID() + "'";
        QueueReceiver replyReceiver= this.getSession().createReceiver(this.getReplyQueue(), filter);

        Message replyMessage = replyReceiver.receive(33000);    // Waiting the reply for maximum 33 sec.
        if (replyMessage == null)
        {
            System.out.println("Destination not responding");
        } else {
            if (replyMessage instanceof ObjectMessage)
            {
                if (replyMessage.getStringProperty("MSG_TYPE").equals(Enums.MessageType.INSTRUMENT_LIST.toString()) &&
                        replyMessage.getStringProperty("USER_ID").equals(control.getUserID()))
                {
                    ObjectMessage replyObject = (ObjectMessage)replyMessage;
                    InstrumentList replyInstrumentList = (InstrumentList)replyObject.getObject();

                    return replyInstrumentList;
                }
            } else {
                throw new IllegalArgumentException("Invalid message type");
            }
        }

        return null;
    }
}

