/**
 * Title:       Control queue receiver with Price reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   PriceReceiverWithReply.java
 * Created:     December 11, 2010
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Control queue receiver with reply mechanism, within OpenMQ broker by extending AbstractQueueReceiver class
 */

package asets.api;

import javax.jms.*;


public class PriceReceiverWithReply extends AbstractQueueReceiver
{
    public PriceReceiverWithReply(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, queueName, userID);
    }

    // Receive messages from Control queue
    public void onMessage(Message message)
    {
        try
        {
            if (message instanceof ObjectMessage)
            {
                if (message.getStringProperty("MSG_TYPE").equals(Enums.MessageType.CONTROL.toString()) &&
                        message.getStringProperty("CTRL_TYPE").equals(Enums.ControlType.PRICE_REQUEST.toString()))
                {
                    ObjectMessage objectMessage = (ObjectMessage)message;

                    // Process Control request and suply Price reply
                    Price replyPrice = processPriceRequest((Control)objectMessage.getObject());

                    // Construct the reply message as an ObjectMessage (Price)
                    ObjectMessage replyMessage = this.getSession().createObjectMessage();
                    replyMessage.setObject(replyPrice);

                    replyMessage.setStringProperty("MSG_TYPE", Enums.MessageType.PRICE.toString());
                    replyMessage.setStringProperty("USER_ID", message.getStringProperty("USER_ID"));

                    replyMessage.setJMSCorrelationID(message.getJMSMessageID());

                    // Create the sender and send Price
                    QueueSender replySender = this.getSession().createSender((Queue)message.getJMSReplyTo());
                    replySender.send(replyMessage);
                }
            }
        } catch (JMSException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected Price processPriceRequest(Control control) throws Exception
    {
        Price replyPrice = new Price();

        // TO DO in the the derived class

        return replyPrice;
    }
}
