/**
 * Title:       Control queue receiver with User reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   TradeListReceiverWithReply.java
 * Created:     December 14, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Control queue receiver with reply mechanism, within OpenMQ broker by extending AbstractQueueReceiver class
 */

package asets.api;

import javax.jms.*;


public class TradeListReceiverWithReply extends AbstractQueueReceiver
{
    public TradeListReceiverWithReply(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, queueName, userID);
    }

    // Receive messages from Control queue
    public void onMessage(Message message)
    {
        try
        {
            if (message instanceof ObjectMessage)
            {
                if (message.getStringProperty("MSG_TYPE").equals(Enums.MessageType.CONTROL.toString()) &&
                        message.getStringProperty("CTRL_TYPE").equals(Enums.ControlType.TRADE_LIST_REQUEST.toString()))
                {
                    ObjectMessage objectMessage = (ObjectMessage)message;

                    // Process Control request and suply TradeList reply
                    TradeList replyTradeList = processTradeListRequest((Control)objectMessage.getObject());

                    // Construct the reply message as an ObjectMessage (TradeList)
                    ObjectMessage replyMessage = this.getSession().createObjectMessage();
                    replyMessage.setObject(replyTradeList);

                    replyMessage.setStringProperty("MSG_TYPE", Enums.MessageType.TRADE_LIST.toString());
                    replyMessage.setStringProperty("USER_ID", message.getStringProperty("USER_ID"));

                    replyMessage.setJMSCorrelationID(message.getJMSMessageID());

                    // Create the sender and send TradeList
                    QueueSender replySender = this.getSession().createSender((Queue)message.getJMSReplyTo());
                    replySender.send(replyMessage);
                }
            }
        } catch (JMSException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected TradeList processTradeListRequest(Control control) throws Exception
    {
        TradeList replyTradeList = new TradeList();

        // TO DO in the the derived class

        return replyTradeList;
    }
}
