/**
 * Title:       User queue receiver with reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   UserReceiverWithReply.java
 * Created:     November 30, 2010
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an User queue receiver with reply mechanism, within OpenMQ broker by extending AbstractQueueReceiver class
 */

package asets.api;

import javax.jms.*;


public class UserReceiverWithReply extends AbstractQueueReceiver
{
    public UserReceiverWithReply(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, queueName, userID);
    }

    // Receive messages from User queue
    public void onMessage(Message message)
    {
        try
        {
            if (message instanceof ObjectMessage)
            {
                if (message.getStringProperty("MSG_TYPE").equals(Enums.MessageType.USER.toString()))
                {
                    ObjectMessage objectMessage = (ObjectMessage)message;

                    // Create the sender and send User
                    QueueSender replySender = this.getSession().createSender((Queue)message.getJMSReplyTo());
                    ObjectMessage replyMessage = this.getSession().createObjectMessage();

                    // Process User request and suply User reply
                    User replyUser = null;
                    if (message.getStringProperty("CTRL_TYPE").equals(Enums.ControlType.USER_REQUEST.toString()))
                        replyUser = processUserRequest((User)objectMessage.getObject());
                    if (message.getStringProperty("CTRL_TYPE").equals(Enums.ControlType.USER_ADD_REQUEST.toString()))
                        replyUser = processAddUserRequest((User)objectMessage.getObject());
                    if (message.getStringProperty("CTRL_TYPE").equals(Enums.ControlType.USER_UPDATE_REQUEST.toString()))
                        replyUser = processUpdateUserRequest((User)objectMessage.getObject());

                    if( replyUser == null ) return;
                    
                    // Construct the reply message as an ObjectMessage (User)
                    replyMessage.setObject(replyUser);

                    replyMessage.setStringProperty("MSG_TYPE", Enums.MessageType.USER.toString());
                    replyMessage.setStringProperty("USER_ID", message.getStringProperty("USER_ID"));
                    replyMessage.setJMSCorrelationID(message.getJMSMessageID());

                    replySender.send(replyMessage);
                }
            }
        } catch (JMSException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected User processUserRequest(User user) throws Exception
    {
        User replyUser = new User();

        // TO DO in the the derived class

        return replyUser;
    }

    protected User processAddUserRequest(User user) throws Exception
    {
        User replyUser = new User();

        // TO DO in the the derived class

        return replyUser;
    }

    protected User processUpdateUserRequest(User user) throws Exception
    {
        User replyUser = new User();

        // TO DO in the the derived class

        return replyUser;
    }
}
