/**
 * Title:       ESE (Exchange Simulation Engine) for ASETS project (ASE Trading System)
 * ClassName:   ESE.java
 * Created:     November 30, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements ESE (Exchange Simulation Engine) class for ASETS API testing
 */

package asets.testapi;

import asets.api.*;
import javax.jms.*;
import java.io.*;
import java.util.concurrent.ConcurrentLinkedQueue;


public class ESE
{
    public static void main(String[] args)
            throws IOException, JMSException, InterruptedException, Exception
    {
        // if (args.length != 1)
        // {
        //     System.out.println("User name missing ... exiting");
        //     System.exit(0);// exit program
        // }
        // TODO code application logic here
        // String userID = args[0];
        String userID = "ESE-MOCK";

        ESEOrderReceiverNoReply orderReceiver =
                new ESEOrderReceiverNoReply("http://asets.ase.ro:8080/imqhttp/tunnel",
                Enums.QueueName.CLIENT_ORDER_QUEUE_MOCK.toString(),
                userID + "-ClientOrder");
        ESEExecutionSenderNoReply execSender =
                new ESEExecutionSenderNoReply("http://asets.ase.ro:8080/imqhttp/tunnel",
                Enums.QueueName.MARKET_EXECUTION_QUEUE_MOCK.toString(),
                userID + "-MarketExecution");
        ESEOrderSenderNoReply orderSender =
                new ESEOrderSenderNoReply("http://asets.ase.ro:8080/imqhttp/tunnel",
                Enums.QueueName.MARKET_ORDER_QUEUE_MOCK.toString(),
                userID + "-MarketOrder");

        try
        {
            System.out.println("Reading...");

            // Read from command line
            // BufferedReader commandLine = new java.io.BufferedReader(new InputStreamReader(System.in));

            // Loop until the word "exit" is typed
            // Type anything else in order to process Execution queue
            while(true)
            {
                // String s = commandLine.readLine();

                // Process first the outstanding Order queue
                ConcurrentLinkedQueue<Order> ordersToSend = orderReceiver.getOrdersToSend();
                while(!ordersToSend.isEmpty())
                {
                    Order localOrder = ordersToSend.poll();
                    orderSender.sendOrder(localOrder);
                }

                // And then the outstanding Execution queue
                ConcurrentLinkedQueue<Execution> executionsToSend = orderReceiver.getExecutionsToSend();
                while(!executionsToSend.isEmpty())
                {
                    Execution localExec = executionsToSend.poll();
                    execSender.sendExecution(localExec);
                }

                // Attempt to take a breath of 11 milliseconds
                try {
                    Thread.sleep(11);
                } catch (InterruptedException e) {
                    //We've been interrupted
                    e.printStackTrace();
                }

                // if (s.equalsIgnoreCase("exit"))
                // {
                //    orderReceiver.close();
                //    execSender.close();
                //    orderSender.close();
                //    System.exit(0);
                // }
            }
        // } catch (IOException e) {
        //     e.printStackTrace();
        } catch (JMSException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try
            {
                orderReceiver.close();
                execSender.close();
                orderSender.close();
            } catch (JMSException e) {
                e.printStackTrace();
            }
            System.exit(0);
        }
    }
}

