/**
 * Title:       Execution queue sender for ESE (Exchange Simulation Engine), within ASETS project (ASE Trading System)
 * ClassName:   ESEExecutionSenderNoReply.java
 * Created:     November 30, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Execution queue sender by extending ExecutionSenderNoReply class
 */

package asets.testapi;

import asets.api.*;
import javax.jms.*;


public class ESEExecutionSenderNoReply extends ExecutionSenderNoReply
{
    public ESEExecutionSenderNoReply(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, queueName, userID);
    }

    // Create and send an Execution message
    @Override
    public void sendExecution(Execution execution) throws JMSException
    {
        // Construct the message as an ObjectMessage (Execution)
        ObjectMessage message = this.getSession().createObjectMessage();
        message.setObject(execution);

        message.setStringProperty("MSG_TYPE", Enums.MessageType.EXECUTION.toString());

        System.out.println("ESE GENERATED EXECUTION - SENT TO OMS");
        System.out.println("User ID: " + execution.getUserID());
        System.out.println("Symbol: " + execution.getSymbol());
        System.out.println("Side: " + execution.getExecSide());
        System.out.println("Quantity: " + execution.getExecQuant());
        System.out.println("Price: " + execution.getExecPrice());
        System.out.println();

        this.getSender().send(message);
    }
}

