/**
 * Title:       OMS (Order MAnagement Server) for ASETS project (ASE Trading System)
 * ClassName:   OMS.java
 * Created:     November 30, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements OMS (Order Management Server) class for ASETS API testing
 */

package asets.testapi;

import asets.api.*;
import javax.jms.*;
import java.io.*;
import java.util.concurrent.ConcurrentLinkedQueue;


public class OMS
{
    public static void main(String[] args) 
            throws IOException, JMSException, InterruptedException, Exception
    {
        // if (args.length != 1)
        // {
        //     System.out.println("User name missing ... exiting");
        //     System.exit(0);// exit program
        // }
        // TODO code application logic here
        // String userID = args[0];
        String userID = "OMS-MOCK";

        OMSOrderReceiverWithReply clientOrderReceiver =
                new OMSOrderReceiverWithReply("http://asets.ase.ro:8080/imqhttp/tunnel",
                Enums.QueueName.ORDER_REQUEST_QUEUE_MOCK.toString(),
                userID + "-Order");
        OMSOrderSenderNoReply clientOrderSender =
                new OMSOrderSenderNoReply("http://asets.ase.ro:8080/imqhttp/tunnel",
                Enums.QueueName.CLIENT_ORDER_QUEUE_MOCK.toString(),
                userID);
        OMSExecutionReceiverNoReply execReceiver =
                new OMSExecutionReceiverNoReply("http://asets.ase.ro:8080/imqhttp/tunnel",
                Enums.QueueName.MARKET_EXECUTION_QUEUE_MOCK.toString(),
                userID + "-MarketExecution");
        OMSExecutionPublisher execPublisher =
                new OMSExecutionPublisher("http://asets.ase.ro:8080/imqhttp/tunnel",
                Enums.TopicName.CLIENT_EXECUTION_TOPIC_MOCK.toString(),
                userID + "-ClientExecution");
        OMSMarketOrderReceiverNoReply marketOrderReceiver =
                new OMSMarketOrderReceiverNoReply("http://asets.ase.ro:8080/imqhttp/tunnel",
                Enums.QueueName.MARKET_ORDER_QUEUE_MOCK.toString(),
                userID + "-MarketOrder");
        OMSOrderPublisher marketOrderPublisher =
                new OMSOrderPublisher("http://asets.ase.ro:8080/imqhttp/tunnel",
                Enums.TopicName.CLIENT_ORDER_TOPIC_MOCK.toString(),
                userID + "-ClientOrder");

        try
        {
            System.out.println("Reading...");

            // Read from command line
            // BufferedReader commandLine = new java.io.BufferedReader(new InputStreamReader(System.in));

            // Loop until the word "exit" is typed
            // Type anything else in order to process Order and Execution queues
            while(true)
            {
                // String s = commandLine.readLine();

                // Process first the outstanding Order queue (for the market)
                ConcurrentLinkedQueue<Order> ordersForMarket = clientOrderReceiver.getOrdersForMarket();
                while(!ordersForMarket.isEmpty())
                {
                    Order clientOrder = ordersForMarket.poll();
                    clientOrderSender.sendOrder(clientOrder);
                }

                // Then the outstanding Order queue (updates for the investors)
                ConcurrentLinkedQueue<Order> ordersToPublish = marketOrderReceiver.getOrdersToPublish();
                while(!ordersToPublish.isEmpty())
                {
                    Order marketOrder = ordersToPublish.poll();
                    marketOrderPublisher.publishOrder(marketOrder);
                }
 
                // And then the outstanding Execution queue (for the investors)
                ConcurrentLinkedQueue<Execution> executionsToPublish = execReceiver.getExecutionsToPublish();
                while(!executionsToPublish.isEmpty())
                {
                    Execution localExec = executionsToPublish.poll();
                    execPublisher.publishExecution(localExec);
                }

                // Attempt to take a breath of 11 milliseconds
                try {
                    Thread.sleep(11);
                } catch (InterruptedException e) {
                    //We've been interrupted
                    e.printStackTrace();
                }

                // if (s.equalsIgnoreCase("exit"))
                // {
                //     clientOrderReceiver.close();
                //     clientOrderSender.close();
                //     execReceiver.close();
                //     execPublisher.close();
                //     marketOrderReceiver.close();
                //     marketOrderPublisher.close();
                //     System.exit(0);
                // }
            }
        // } catch(IOException e) {
        //     e.printStackTrace();
        } catch (JMSException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try
            {
                clientOrderReceiver.close();
                clientOrderSender.close();
                execPublisher.close();
                execReceiver.close();
                marketOrderReceiver.close();
                marketOrderPublisher.close();
            } catch (JMSException e) {
                e.printStackTrace();
            }
            System.exit(0);
        }
    }
}

