/**
 * Title:       Execution topic publisher for OMS, within ASETS project (ASE Trading System)
 * ClassName:   OMSExecutionPublisher.java
 * Created:     November 30, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Execution topic publisher for OMS, by extending ExecutionPublisher class
 */

package asets.testapi;

import asets.api.*;
import javax.jms.*;


public class OMSExecutionPublisher extends ExecutionPublisher
{
    public OMSExecutionPublisher(String imqAddress, String topicName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, topicName, userID);
    }

    // Create and publish an Execution message
    @Override
    public void publishExecution(Execution execution) throws JMSException
    {
        ObjectMessage message = this.getSession().createObjectMessage();
        message.setObject(execution);

        message.setStringProperty("MSG_TYPE", Enums.MessageType.EXECUTION.toString());
        message.setStringProperty("USER_ID", execution.getUserID() + "-Execution");

        System.out.println("OMS PUBLISHED EXECUTION");
        System.out.println("User ID: " + execution.getUserID());
        System.out.println("Symbol: " + execution.getSymbol());
        System.out.println("Side: " + execution.getExecSide());
        System.out.println("Quantity: " + execution.getExecQuant());
        System.out.println("Price: " + execution.getExecPrice());
        System.out.println();

        this.getPublisher().publish(message);
    }
}