/**
 * Title:       Execution queue receiver for OMS, within ASETS project (ASE Trading System)
 * ClassName:   OMSExecutionReceiverNoReply.java
 * Created:     November 30, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Execution queue receiver for OMS, by extending ExecutionReceiverNoReply class
 */

package asets.testapi;

import asets.api.*;
import javax.jms.*;
import java.util.concurrent.ConcurrentLinkedQueue;

public class OMSExecutionReceiverNoReply extends ExecutionReceiverNoReply
{
    private ConcurrentLinkedQueue<Execution> executionsToPublish = null;

    public OMSExecutionReceiverNoReply(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, queueName, userID);

        executionsToPublish  = new ConcurrentLinkedQueue<Execution>();
    }

    public ConcurrentLinkedQueue<Execution> getExecutionsToPublish()
    {
        return this.executionsToPublish;
    }

    @Override
    protected void processExecution(Execution execution) throws Exception
    {
        System.out.println("OMS RECEIVED EXECUTION FROM ESE");
        System.out.println("User ID: " + execution.getUserID());
        System.out.println("Symbol: " + execution.getSymbol());
        System.out.println("Side: " + execution.getExecSide());
        System.out.println("Quantity: " + execution.getExecQuant());
        System.out.println("Price: " + execution.getExecPrice());
        System.out.println();

        // Publish the received Execution to the clients
        executionsToPublish.offer(execution);
    }
}
