/**
 * Title:       Order queue receiver for OMS, within ASETS project (ASE Trading System)
 * ClassName:   OMSOrderReceiverNoReply.java
 * Created:     January 23, 2010
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Order queue receiver for OMS, by extending OrderReceiverNoReply class
 */

package asets.testapi;

import asets.api.*;
import javax.jms.*;
import java.util.concurrent.ConcurrentLinkedQueue;

public class OMSMarketOrderReceiverNoReply extends OrderReceiverNoReply
{
    private ConcurrentLinkedQueue<Order> ordersToPublish = null;

    public OMSMarketOrderReceiverNoReply(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, queueName, userID);

        ordersToPublish  = new ConcurrentLinkedQueue<Order>();
    }

    public ConcurrentLinkedQueue<Order> getOrdersToPublish()
    {
        return this.ordersToPublish;
    }

    @Override
    protected void processOrder(Order order) throws Exception
    {
        System.out.println("OMS RECEIVED UPDATE ORDER FROM ESE");
        System.out.println("User ID: " + order.getUserID());
        System.out.println("Symbol: " + order.getSymbol());
        System.out.println("Order Type: " + order.getOrderType().toString());
        System.out.println("Side: " + order.getOrderSide().toString());
        System.out.println("Quantity: " + order.getOrderQuant());
        System.out.println("Price: " + order.getOrderPrice());
        System.out.println("Order Status: " + order.getOrderStatus().toString());
        System.out.println("Market Status: " + order.getMarketStatus().toString());
        System.out.println();
 
        // Publish the received Order to the clients
        ordersToPublish.offer(order);
    }
}
