/**
 * Title:       Order topic publisher for OMS, within ASETS project (ASE Trading System)
 * ClassName:   OMSOrderPublisher.java
 * Created:     January 23, 2010
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Order topic publisher for OMS, by extending OrderPublisher class
 */

package asets.testapi;

import asets.api.*;
import javax.jms.*;


public class OMSOrderPublisher extends OrderPublisher
{
    public OMSOrderPublisher(String imqAddress, String topicName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, topicName, userID);
    }

    // Create and publish an Order message
    @Override
    public void publishOrder(Order order) throws JMSException
    {
        ObjectMessage message = this.getSession().createObjectMessage();
        message.setObject(order);

        message.setStringProperty("MSG_TYPE", Enums.MessageType.ORDER.toString());
        message.setStringProperty("USER_ID", order.getUserID() + "-Order");

        System.out.println("OMS PUBLISHED ORDER");
        System.out.println("User ID: " + order.getUserID());
        System.out.println("Symbol: " + order.getSymbol());
        System.out.println("Order Type: " + order.getOrderType().toString());
        System.out.println("Side: " + order.getOrderSide().toString());
        System.out.println("Quantity: " + order.getOrderQuant());
        System.out.println("Price: " + order.getOrderPrice());
        System.out.println("Order Status: " + order.getOrderStatus().toString());
        System.out.println("Market Status: " + order.getMarketStatus().toString());
        System.out.println();

        this.getPublisher().publish(message);
    }
}