/**
 * Title:       Order queue receiver with reply mechanism for OMS, within ASETS project (ASE Trading System)
 * ClassName:   OMSOrderReceiverWithReply.java
 * Created:     November 30, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an OMS Order queue receiver with reply mechanism, by extending OrderReceiverWithReply class
 */

package asets.testapi;

import asets.api.*;
import javax.jms.*;
import java.util.concurrent.ConcurrentLinkedQueue;


public class OMSOrderReceiverWithReply extends OrderReceiverWithReply
{
    private ConcurrentLinkedQueue<Order> ordersForMarket = null;

    public OMSOrderReceiverWithReply(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, queueName, userID);

        ordersForMarket = new ConcurrentLinkedQueue<Order>();
    }

    public ConcurrentLinkedQueue<Order> getOrdersForMarket()
    {
        return this.ordersForMarket;
    }

    @Override
    protected Order processOrderRequest(Order order) throws Exception
    {
        Order replyOrder = new Order();

        System.out.println("OMS ORDER RECEIVED FROM TUI");
        System.out.println("User ID: " + order.getUserID());
        System.out.println("Order Type: " + order.getOrderType().toString());
        System.out.println("Symbol: " + order.getSymbol());
        System.out.println("Side: " + order.getOrderSide().toString());
        System.out.println("Quantity: " + order.getOrderQuant());
        System.out.println("Price: " + order.getOrderPrice());
        System.out.println("Order Status: " + order.getOrderStatus().toString());
        System.out.println("Market Status: " + order.getMarketStatus().toString());
        System.out.println();

        order.setOrderStatus(Enums.OrderStatus.ADDED);
        order.setMarketStatus(Enums.MarketStatus.PENDING);

        // Publish Order to the market
        ordersForMarket.offer(order);

        replyOrder.setUserID(order.getUserID());
        replyOrder.setOrderType(order.getOrderType());
        replyOrder.setSymbol(order.getSymbol());
        replyOrder.setOrderSide(order.getOrderSide());
        replyOrder.setOrderQuant(order.getOrderQuant());
        replyOrder.setOrderPrice(order.getOrderPrice());
        replyOrder.setOrderStatus(order.getOrderStatus());
        replyOrder.setMarketStatus(order.getMarketStatus());

        System.out.println("OMS ORDER REPLIED TO TUI");
        System.out.println("User ID: " + replyOrder.getUserID());
        System.out.println("Order Type: " + replyOrder.getOrderType().toString());
        System.out.println("Symbol: " + replyOrder.getSymbol());
        System.out.println("Side: " + replyOrder.getOrderSide().toString());
        System.out.println("Quantity: " + replyOrder.getOrderQuant());
        System.out.println("Price: " + replyOrder.getOrderPrice());
        System.out.println("Order Status: " + replyOrder.getOrderStatus().toString());
        System.out.println("Market Status: " + replyOrder.getMarketStatus().toString());
        System.out.println();

        return replyOrder;
    }
}
