/**
 * Title:       Order queue sender for OMS, within ASETS project (ASE Trading System)
 * ClassName:   OMSOrderSenderNoReply.java
 * Created:     January 20, 2010
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Order queue sender, within OpenMQ broker, by extending AbstractQueueSenderNoReply class
 */

package asets.testapi;

import asets.api.*;
import javax.jms.*;


public class OMSOrderSenderNoReply extends OrderSenderNoReply
{
    public OMSOrderSenderNoReply(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, queueName, userID);
    }

    // Create and send an Order message
    @Override
    public void sendOrder(Order order) throws JMSException
    {
        // Construct the message as an ObjectMessage (Order)
        ObjectMessage message = this.getSession().createObjectMessage();
        message.setObject(order);

        message.setStringProperty("MSG_TYPE", Enums.MessageType.ORDER.toString());
        // Market Orders are received either from OMS or from PROG
        // OMS      - Order Management Server
        // PROG     - Pseudo Random Order Generator
        message.setStringProperty("USER_ID", this.getUserID());

        order.setMarketStatus(Enums.MarketStatus.ON_MARKET);

        System.out.println("OMS SENT ORDER TO THE MARKET");
        System.out.println("User ID: " + order.getUserID());
        System.out.println("Order Type: " + order.getOrderType().toString());
        System.out.println("Symbol: " + order.getSymbol());
        System.out.println("Side: " + order.getOrderSide().toString());
        System.out.println("Quantity: " + order.getOrderQuant());
        System.out.println("Price: " + order.getOrderPrice());
        System.out.println("Order Status: " + order.getOrderStatus().toString());
        System.out.println("Market Status: " + order.getMarketStatus().toString());
        System.out.println();

        this.getSender().send(message);
    }
}

