/**
 * Title:       TUI (Trading User Interface) for ASETS project (ASE Trading System)
 * ClassName:   TUI.java
 * Created:     November 30, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements TUI (Trading User Interface) class for ASETS API testing
 */

package asets.testapi;

import asets.api.*;
import javax.jms.*;
import java.io.*;
import java.util.StringTokenizer;


public class TUI
{
    public static void main(String[] args) throws IOException, JMSException, Exception
    {
        // if (args.length != 1)
        // {
        //     System.out.println("User name missing ... exiting");
        //     System.exit(0);// exit program
        // }
        // TODO code application logic here
        // String userID = args[0];
        String userID = "TUI-MOCK";

        TUIOrderSenderWithReply orderSender =
                new TUIOrderSenderWithReply("http://asets.ase.ro:8080/imqhttp/tunnel",
                Enums.QueueName.ORDER_REQUEST_QUEUE_MOCK.toString(),
                Enums.QueueName.ORDER_REPLY_QUEUE_MOCK.toString(),
                userID);
        TUIExecutionSubscriber execSubscriber =
                new TUIExecutionSubscriber("http://asets.ase.ro:8080/imqhttp/tunnel",
                Enums.TopicName.CLIENT_EXECUTION_TOPIC_MOCK.toString(),
                userID + "-Execution");
        TUIOrderSubscriber orderSubscriber =
                new TUIOrderSubscriber("http://asets.ase.ro:8080/imqhttp/tunnel",
                Enums.TopicName.CLIENT_ORDER_TOPIC_MOCK.toString(),
                userID + "-Order");

        BufferedReader stdin = new BufferedReader(new InputStreamReader(System.in));
        System.out.println("Enter new order. Press ENTER to quit application.");
        System.out.println("Usage syntax: <Symbol>, <Quantity>, <Price>");
        System.out.println("Example: BRD, 2000, 13.68");
        
        try
        {
            while(true)
            {
                String newOrderRequest = stdin.readLine();
                if ( (newOrderRequest == null) || (newOrderRequest.trim().length() <= 0))
                {
                    orderSender.close();
                    execSubscriber.close();
                    orderSubscriber.close();
                    System.exit(0);
                } else {
                    StringTokenizer st = new StringTokenizer(newOrderRequest, ",");
                    String symbol = String.valueOf(st.nextToken().trim());
                    long quantity = Long.valueOf(st.nextToken().trim()).longValue();
                    float price = Float.valueOf(st.nextToken().trim()).floatValue();

                    Order newOrder = new Order();

                    newOrder.setUserID(userID);
                    newOrder.setOrderType(Enums.OrderType.LIMIT_PRICE);
                    newOrder.setOrderSide(Enums.Side.BUY);
                    newOrder.setSymbol(symbol);
                    newOrder.setOrderQuant(quantity);
                    newOrder.setOrderPrice(price);
                    newOrder.setOrderStatus(Enums.OrderStatus.ADD);
                    newOrder.setMarketStatus(Enums.MarketStatus.PENDING);

                    System.out.println("TUI ORDER SENT TO OMS");
                    System.out.println("User ID: " + newOrder.getUserID());
                    System.out.println("Order Type: " + newOrder.getOrderType().toString());
                    System.out.println("Symbol: " + newOrder.getSymbol());
                    System.out.println("Side: " + newOrder.getOrderSide().toString());
                    System.out.println("Quantity: " + newOrder.getOrderQuant());
                    System.out.println("Price: " + newOrder.getOrderPrice());
                    System.out.println("Order Status: " + newOrder.getOrderStatus().toString());
                    System.out.println("Market Status: " + newOrder.getMarketStatus().toString());
                    System.out.println();

                    Order replyOrder = orderSender.sendOrderGetReply(newOrder);                   
                    if (replyOrder == null)
                    {
                        System.out.println("NO REPLY FROM OMS");
                        System.out.println();
                    } else {
                        System.out.println("TUI RECEIVED ORDER REPLY FROM OMS");
                        System.out.println("User ID: " + replyOrder.getUserID());
                        System.out.println("Order Type: " + replyOrder.getOrderType().toString());
                        System.out.println("Symbol: " + replyOrder.getSymbol());
                        System.out.println("Side: " + replyOrder.getOrderSide().toString());
                        System.out.println("Quantity: " + replyOrder.getOrderQuant());
                        System.out.println("Price: " + replyOrder.getOrderPrice());
                        System.out.println("Order Status: " + replyOrder.getOrderStatus().toString());
                        System.out.println("Market Status: " + replyOrder.getMarketStatus().toString());
                        System.out.println();
                    }
                }
            }
        } catch(IOException e) {
            e.printStackTrace();
        } catch (JMSException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try
            {
                orderSender.close();
                execSubscriber.close();
                orderSubscriber.close();
            } catch (JMSException e) {
                e.printStackTrace();
            }
            System.exit(0);
        }
    }
}
